#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>
#include <stdio.h>
#include <stdlib.h>
#include <string>

using namespace std;

int main(void)
{
	cout << "Reading parameters for runoff simulation." << endl;
	ifstream finParam("../param-event-ver222.dat");
	if(!finParam){
		cout << "../param-event-ver222.dat cannot open." << endl;
		return 1;
	}

	// vZ̈ simulation range
	cout << "1) Reading parameters for study area." << endl;

	int wTopoScale;

	finParam >> wTopoScale;
	cout << "  Topography scale : " << wTopoScale << endl;

	int wCol,wRow;
	float wSWLON, wSWLAT;

	finParam >> wCol; 
	finParam >> wRow; 
	cout << "  Number of col : " << wCol << "  Number of row : " << wRow << endl;

	finParam >> wSWLON; 
	finParam >> wSWLAT;
	cout << " Sorth West corner Longitude (degree) : " << wSWLON 
		 << " Latitude (deg) : " << wSWLAT << endl;

	const int TopoScale = wTopoScale;  // Topography scale
	const int Col = wCol;     // column number of study area
	const int Row = wRow;     // row number of study area
	const float SWLON = wSWLON; // South west corner of longitude (degree)
	const float SWLAT = wSWLAT; // South west corner of latiitude (degree)

	// ̓f[^̈ input data range
	cout << "2) Reading parameters for input data." << endl;

	int wROFtype;	// added by Tanaka T. 2018/6/28
	finParam >> wROFtype;
	cout << "Read ";
	if(wROFtype == 1){
		cout << "./input/rain.dat" << endl;
	}
	else if(wROFtype == 2){
		cout << "./input/rain.bin" << endl;
	}
	else{
		cout << "Set rainfall data format (1: text, 2: 4 byte binary)" << endl;
		exit(1);
	}
	const int ROFtype = wROFtype;
	
	int wROFnum_col, wROFnum_row;
	double wDelta_x, wDelta_y, wROF_WLON, wROF_SLAT;
	finParam >> wROFnum_col;
	finParam >> wROFnum_row;
	cout << "  Number of col : " << wROFnum_col 
		 << "  Number of row : " << wROFnum_row << endl;

	finParam >> wDelta_x;
	finParam >> wDelta_y;
	cout << "  Input data grid size of x-direction : " << wDelta_x << endl;
	cout << "  Input data grid size of y-direction : " << wDelta_y << endl;

	finParam >> wROF_WLON; 
	cout << "  Longitude of the center of the Southwest corner of input data (degree) : " << wROF_WLON << endl;
	finParam >> wROF_SLAT; 
	cout << "  Latitude of the center of the Southwest corner of input data (degree) : " << wROF_SLAT << endl;
													
	const int ROFnum_col = wROFnum_col;
	const int ROFnum_row = wROFnum_row; 
	const float Delta_x = wDelta_x;
	const float Delta_y = wDelta_y;
	const float ROF_WLON = wROF_WLON;
	const float ROF_SLAT = wROF_SLAT;

	// vZ simulation term
	cout << "3) Reading parameters for simuration duration." << endl;
	int wNdays;
	finParam >> wNdays;
	cout << "  Simulation duration (days) : " << wNdays << endl;

	const int Ndays = wNdays; // Simulation duration (days)

	finParam.close();
	//--------------------- end parameter reading ------------------------------------------------

    // setting column and row number
	cout << "Input column number to extract data!" << endl;
	cout << "Column number begins from 1 from east." << endl;
	cin >> wCol;

	string ss;
    if (wCol > Col || wCol < 0) {
		cout << "Column number is out of range!" << endl;
		return 1;
	}

	cout << "Input row number to extract data!" << endl;
	cout << "Row number begins from 1 from south." << endl;
	cin >> wRow;
    if (wRow > Row || wRow < 0) {
		cout << "Row number is out of range!" << endl;
		return 1;
	}

	cout << "Column number : " << wCol << " Row number : " << wRow << endl;

	// Hourly output data open
	char DischargeFileName[256];
	ifstream FileHourly; // Hourly data

	//sprintf_s(DischargeFileName, 256, "./dischargeHourly.bin");
	sprintf(DischargeFileName, "./dischargeHourly.bin");

	cout << "Read " << DischargeFileName << endl;
	FileHourly.open(DischargeFileName, ios::in | ios::binary);

	if(!FileHourly){
		cout << "Discharge data cannot open." << endl;
		return 1;
	}

	int max_time_counter = Ndays*24;
	float discharge;




	//Check acc number of the basin 11/28/2014 Tanaka
	/*--------------------------------------------------------------------------------*/
	int basin;
	sprintf(DischargeFileName, "../topoData/flowAcc.bin");
	cout << "Read " << DischargeFileName << endl;
	ifstream FileAcc; // Daily data
	FileAcc.open(DischargeFileName, ios::in | ios::binary);
	if(!FileAcc){
		cout << "Flow accumulation data cannot open." << endl;
		return 1;
	}
	for (int j = 1; j <= Row; j++) {
		for (int k = 1; k <= Col; k++) {
			FileAcc.read((char*)&basin, sizeof(int));
			if (j == wRow && k == wCol) {
				cout << endl;
				cout << "Flow accumulation (not the exactly same as area) : " << basin << endl;
				cout << endl;
				break;
			}
		}
	}
	FileAcc.close();
	/*--------------------------------------------------------------------------------*/


	ofstream outFile;
	//sprintf_s(DischargeFileName, 256, "./%d-%d_discharge.dat", wCol, wRow);
	sprintf(DischargeFileName, "./%d-%d_discharge.dat", wCol, wRow);
	cout << "Write " << DischargeFileName << "\r";

	outFile.open(DischargeFileName, ios::out | ios::binary);
	if(!outFile){
		cout << "Time series discharge output file cannot open." << endl;
		return 1;
	}

	for (int i = 1; i <= max_time_counter; i++) {
		for (int j = 1; j <= Row; j++) {
			for (int k = 1; k <= Col; k++) {
				FileHourly.read((char*)&discharge, sizeof(float));
				if (j == wRow && k == wCol) {
					outFile << i << " " << discharge << endl;
				}
			}
		}

	}

	FileHourly.close();
	outFile.close();
	
}

